;				**********************************
;				*                                *
;				*     Sound Sample Play Back     *
;				*                                *
;				*   AVR AT90S8515, 8MHz Clock    *
;				*                                *
;				*          28-Oct-2002           *
;				*    Version 1.00, AVRStudio4    *
;				*                                *
;				*       By Anthony Barrett       *
;				*                                *
;				**********************************
;
;
; This program will play back sound sample data generated by
; "Wave2AVR.exe".  See application note for details.
;
; Port pin descriptions:
;
;  PB0 -> Replay button
;  PB1 -> Transducer/Speaker output


;Includes:

.include "m16def.inc"


;Define SRAM variables:

.equ SRAMStart = $0060
.equ SamplePtrL = SRAMStart
.equ SamplePtrH = SamplePtrL + 1
.equ SampleCountL = SamplePtrH + 1
.equ SampleCountH = SampleCountL + 1
.equ SampleBitCount = SampleCountH + 1
.equ SampleByte = SampleBitCount + 1
.equ SampleRate = SampleByte + 1
.equ SampleComplete = SampleRate + 1


;Vector table:
				rjmp	RESET				;Reset handler
.ORG 	OVF0addr
				rjmp	TIM0_OVF

.ORG INT_VECTORS_SIZE
;Initalize:


RESET:			ldi		r16,high(RAMEND) 	;Set stack pointer
				out		SPH,r16
				ldi		r16,low(RAMEND)
				out		SPL,r16

				ldi		r16,0b00000001		;Setup PORTB
				out		PORTB,r16			;PB0 pullup on
				ldi		r16,0b00000010		;PB1 output  
				out		DDRB,r16

				ldi		r16,(1<<CS01)		;Set Timer0 pre-scale to Clk/8
				out		TCCR0,r16

;Start interupts:				

				sei							;Start ints


;Main Progarm loop:
                		
Main:			ldi		ZL,low(2 * Sample)	;Z points to sample data
				ldi		ZH,high(2 * Sample)
				rcall	PlaySample			;Start play back

WaitLoop:		lds		r16,SampleComplete	;Wait for sample to finish
				tst		r16
				breq	WaitLoop

				rjmp	Main				; endlos

WaitLoop2:		sbic	PINB,0				;Wait for button press
				rjmp	WaitLoop2
				rjmp	Main


;Handlers:


TIM0_OVF:		push	r16					;Save r16
				in		r16,SREG			;Save status reg
				push	r16
				
				lds		r16,SampleRate		;Restart Timer0 count 
				out		TCNT0,r16

				lds		r16,SampleByte		;Ouput next bit
				sbrc	r16,0				
				sbi		PORTB,1
				sbrs	r16,0
				cbi		PORTB,1

				ror		r16					;Setup next bit
				sts		SampleByte,r16
				
				lds		r16,SampleBitCount	;Check bit count
				inc		r16
				cpi		r16,8
				brne	T0Exit
				
				push	ZL					;Saved used regs
				push	ZH
				
				lds		ZL,SampleCountL		;Check sample count
				lds		ZH,SampleCountH
				sbiw	ZL,1
				sts		SampleCountL,ZL
				sts		SampleCountH,ZH
				brne	T0GetNextByte

				ldi		r16,0b00000000		;Disable Timer0 ints (stop)
				out		TIMSK,r16
				
				ser		r16					;Signal sample complete
				sts		SampleComplete,r16
				rjmp	T0Done

T0GetNextByte:	push	r0					;Save r0				
								
				lds		ZL,SamplePtrL		;Get next byte
				lds		ZH,SamplePtrH
				adiw	ZL,1
				sts		SamplePtrL,ZL
				sts		SamplePtrH,ZH
				lpm
				sts		SampleByte,r0

				pop		r0					;Restore r0
				
T0Done:			pop		ZH					;Restore used regs				
				pop		ZL

				clr		r16					;Restart bit count
								
T0Exit:			sts		SampleBitCount,r16
				
				pop		r16					;Restore stat reg
				out		SREG,r16
				pop		r16					;Restore r16
				reti


;Subroutines:

PlaySample:		lpm							;Z -> Sample data
				sts		SampleCountL,r0		;Get sample size
				adiw	ZL,1
				lpm
				sts		SampleCountH,r0
				adiw	ZL,1

				lpm							;Get sample rate (TCNT0 value)
				sts		SampleRate,r0
				adiw	ZL,1
				adiw	ZL,1				;Skip high byte

				sts		SamplePtrL,ZL		;Save sample pointer
				sts		SampleptrH,ZH

				clr		r16					;Restart bit count
				sts		SampleBitCount,r16
				
				sts		SampleComplete,r16	;Clear complete flag

				lpm							;Get first byte
				sts		SampleByte,r0
				
				ser		r16					;Restart Timer0 count
				out		TCNT0,r16
				
				ldi		r16,(1<<TOV0)		;Enable Timer0 ints (start)
				out		TIMSK,r16
				ret


;Include sample data file(s) generated by "Wave2AVR.exe":

Sample:

.include "bim.asm"


